# -*- coding: utf-8 -*-

"""
    Acomea exposed apis
    ====

    ATTENZIONE:
     in questo momento le api che vengono esposte sono raggiungibili in produzione esclusivamente via CURL/WGET.
     La libreria standard utilizzata (python-requests) viene rediretta in http e viene ovviamente generato un errore.

     Use the following snippet to retrieve a valid token::

        import requests
        from requests.auth import HTTPBasicAuth
        username = '<username>'
        password = '<password'>
        r = requests.post(
            'http://localhost:8000/api/api-token-auth/',
             auth=HTTPBasicAuth(username, password),
             data=dict(username=username, password=password)
        )

     If using via wget::
        URL='https://www.acomea.it/api/mailup_unsubscribe_acomea_contact/'
        wget -S --post-data='{"recipient_email":"ciao"}' --header=Content-Type:application/json $URL

"""


import logging
from django.conf import settings
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.decorators import api_view
from rest_framework.renderers import JSONRenderer
from rest_framework.parsers import JSONParser

from acomea.contact.models import Contact, Group, Email
from acomea.mailup.utils.client import MailUpClient

logger = logging.getLogger('mailup')


class JSONResponse(HttpResponse):
    """
    An HttpResponse that renders its content into JSON.
    """
    def __init__(self, data, **kwargs):
        content = JSONRenderer().render(data)
        kwargs['content_type'] = 'application/json'
        super(JSONResponse, self).__init__(content, **kwargs)


@csrf_exempt
@api_view(['POST'])
def mailup_unsubscribe_acomea_contact(request, key="acomea2014ws!"):
    """
    Api view waiting for post requests generated by mailup webhook for contact unsubscription.
    We need to change auth_materiale_commerciale flag on our related contact, retrieved via email.
    On Contact save date_last_modify_for_mailup is updated.
    The post request we expect is composed by::

        recipient_email: The email of the recipient modified on mailup;
    """
    logger.debug("### Starting mailup_unsubscribe_acomea_contact ###")
    if request.method == 'POST':
        data = request.DATA
        if not isinstance(request.DATA, dict):
            data = JSONParser().parse(request)
        logger.debug("Data: %r" % data)
        try:
            email = data["recipient_email"].strip()
            contact = Contact.objects.get(emails__email=email)
            if contact:
                contact.auth_materiale_commerciale = False
                contact.save()
                msg = "Contact updated"
                logger.debug(msg)
                return JSONResponse({"msg": msg}, status=201)
            else:
                msg = "Contact not found"
                logger.debug(msg)
                return JSONResponse({"msg": msg}, status=200)
        except Exception, e:
            msg = "Error: Exception %s found" % e
            logger.debug(msg)
            return JSONResponse({"msg": msg}, status=400)
        msg = "Bad data parameters"
        logger.debug(msg)
        return JSONResponse({"msg": msg}, status=400)


@csrf_exempt
@api_view(['POST'])
def mailup_update_acomea_contact_groups(request, key="acomea2014ws!"):
    """
    Api view waiting for post requests generated by mailup webhook for any modify on a recipient in mailup console.
    We just check if recipient groups are changed and, if true, we need to update them in acomea database.
    If a group is not found we need to create it and then associate the contact.
    The post request we expect is composed by::

        recipient_email: The email of the recipient modified on mailup;
        recipient_groups: A list made by recipient groups.
    """
    return JSONResponse({"msg": 'OK'}, status=200)    
    list_id = settings.ACOMEA_MAILUP_LIST_ID
    logger.debug("### Starting mailup_update_acomea_contact_groups ###")
    if request.method == 'POST':
        data = request.DATA
        logger.debug("Data: %r" % data)
        if not isinstance(request.DATA, dict):
            data = JSONParser().parse(request)
        logger.debug("Data: %r" % data)
        mailup_email = data["recipient_email"].strip()
        mailup_first_name = data["recipient_first_name"].strip()
        mailup_last_name = data["recipient_first_name"].strip()
        mailup_groups = [int(group) for group in data["recipient_groups"].split(",") if group]
        logger.debug("Contact Mailup groups: {}".format(mailup_groups))
        try:
            mailup_client = MailUpClient()
            mailup_client.retrieve_access_token()
            read_groups_json = mailup_client.read_groups(list_id)
            contact_exists = Contact.objects.filter(emails__email__iexact=mailup_email).exists()
            if contact_exists:
                logger.debug("Contatto trovato")
                contact = Contact.objects.get(emails__email__iexact=mailup_email)
            else:
                logger.debug("Contatto non trovato")
                contact = Contact.objects.create(
                    first_name=mailup_first_name,
                    last_name=mailup_last_name,
                    bonifica=True,
                    auth_trattamento_dati=True,
                    auth_materiale_commerciale=True,
                )
                Email.objects.create(contact=contact, email=mailup_email, default=True)
            logger.debug("Contatto da sincronizzare: {}".format(contact.id))
            # Controllo stato newsletter contatto
            status_dict = {True: "Subscribed", False: "Unsubscribed"}
            is_subscribed, msg = contact.can_create_message
            db_status = status_dict[is_subscribed]
            is_subscribed_json = mailup_client.check_subscriber_exists(list_id, "Subscribed", mailup_email)
            mailup_status = "Subscribed" if is_subscribed_json["TotalElementsCount"] else "Unsubscribed"
            if db_status != mailup_status:
                if mailup_status == "Subscribed":
                    contact.auth_materiale_commerciale = True
                    logger.debug("Contatto abilitato")
                elif mailup_status == "Unsubscribed":
                    contact.auth_materiale_commerciale = False
                    logger.debug("Contatto disabiliato")
                contact.save()
            else:
                logger.debug("Stato contatto gia' allineato")
            # Controllo gruppi
            for mailup_group_id in mailup_groups:
                try:
                    group = Group.objects.get(mailup_group_id=mailup_group_id)
                except Group.DoesNotExist:
                    mailup_group_id, mailup_group_name = [
                        (group["idGroup"], group["Name"])
                        for group in read_groups_json["Items"] if int(group["idGroup"]) == mailup_group_id
                    ][0]
                    group = Group(name=mailup_group_name, mailup_group_id=mailup_group_id)
                    group.save()
                    logger.debug("New group %s created" % group)
                if group not in contact.groups.all():
                    contact.groups.add(group)  # signal "contact_group_signal" change "first_group" field, to use contact instace remember to reload
                    logger.debug("%s added to group %s" % (mailup_email, group))
                else:
                    logger.debug("%s already in group %s" % (mailup_email, group.name))
            msg = "Contact groups check/update group done"
            logger.debug(msg)
            return JSONResponse({"msg": msg}, status=201)
        except Exception, e:
            msg = "Error: Exception %s found" % e
            logger.debug(msg)
            return JSONResponse({"msg": msg}, status=400)
        msg = "Bad data parameters"
        logger.debug(msg)
        return JSONResponse({"msg": msg}, status=400)

@csrf_exempt
@api_view(['POST'])
def mailup_unsubscribe_newsletter_risparmiamocelo_contact(request):
    """
    Api view waiting for post requests generated by mailup webhook for contact unsubscription.
    We just need to delete the contact from newsletter risparmiamocelo group

        recipient_email: The email of the recipient modified on mailup;
    """
    logger.debug("### Starting mailup_unsubscribe_newsletter_risparmiamocelo_contact ###")
    if request.method == 'POST':
        data = request.DATA
        logger.debug("Data: %r" % data)
        if not isinstance(request.DATA, dict):
            data = JSONParser().parse(request)
        try:
            email = data["recipient_email"].strip()
            try:
                contact = Contact.objects.get(emails__email=email)
                newsletter_risparmiamocelo_group = Group.objects.get(id=settings.NEWSLETTER_RISPARMIAMOCELO_GROUP_ID)
                contact.groups.remove(newsletter_risparmiamocelo_group)
                msg = "Contact removed from newsletter risparmiamocelo group"
                logger.debug(msg)
                return JSONResponse({"msg": msg}, status=201)
            except Contact.DoesNotExist:
                msg = "Contact with email %s not found" % email
                logger.debug(msg)
                return JSONResponse({"msg": msg}, status=400)
        except Exception, e:
            msg = "Error: Exception %s found" % e
            logger.debug(msg)
            return JSONResponse({"msg": msg}, status=400)
        msg = "Bad data parameters"
        logger.debug(msg)
        return JSONResponse({"msg": msg}, status=400)
